/*****************************************************************
* Plugin to collect IDs of players connecting to server.
* It will only store IDs if they don't exist on the server
* already (However, if the same ID but different NAME, then
* ID IS stored. Hence all the players 'fakenicks' will
* be stored along with their IDs).
*
* original idea/author: jugger1@blueyonder.co.uk
*
* Main fixes and slight additions done by Dark.MenacE (darkmenace@cox.net, darkmenace11958@hotmail.com), with
* some help from CyberMind and a few others from #adminmod
*
* -Fixed a bug which logs duplicate lines of the same user.
* -Removed LAN detection, since Steam is online-only. I left the possability to log to lan-only in because Valve
* claims it's working on a lan version of Steam.
* -Removed Error checking, since it was useless and just added to confusion in the code
* -Added extra timer checks
* -Increased the default base max-entries and max-nicknames (namely for servers with high traffic)
* (19/1/2004 - January 19th, 2004) ver 1.3
*
* Fixed duplicate entries bug (Much, much thanks to CyberMind and the guys at #adminmod on irc.gamesnet.net
* Fixed bug where players would not be added when they first connected, but would be added on a map change
* Added a feature to record different ways:
* 0 (Default)-- Only records usernames and SteamIDs.
* 1-- Records SteamIDs and IPs to id_info.txt.
* 2-- Records SteamIDs and IPs to id_info_admin.txt while also records just SteamIDs to id_info.txt.
* (23/11/2003 - November 23rd, 2003) ver 1.2.5
*
* Fixed Auth check with Steam (Thanks, Cybermind!)
* (4/10/2003) ver 1.2.2
*
* Fixed bug in welcome message. Added 'greetings' sound on entry.
* Added fix for blank database entries.
* (2/10/2003) ver 1.2.1
*
* Updated to collect Steam tags.
* (30/9/2003) ver 1.2.0
*
* Corrected Linux issue with comments done using '//' when compiling.
* (9/3/2002) ver 1.1.5
*
* Corrected bug when renaming on reaching MAX_NICKS.
* Added a 'new line' character to each output to the database, as
* it sometimes appended all the names/info of several players
* together on a line; rendering the plugin inoperable.
* (25/2/2002) ver 1.1.4
*
* Added error messaging and WelcomeMsg (0/1) to disable/enable
* welcome message when users connect.
* (14/2/02) ver 1.1.3
*
* After the plugin detects MAX_NICKS names stored, it'll auto
* rename the person concerned to the last name used from the
* database. (8/2/2002) ver 1.1.2
*
* Merged the 2 separate database files in to one: "info_id.ini"
* (or "info_ip.ini" for a LAN server)
* (7/2/2002) ver 1.1.1
*
* Added 'admin_showme' command to allow all names attached to
* a ID or IP to be listed (5/2/2002) ver 1.1.0
*
* Added server detection so that it'll record IP address if
* LAN server detected ('sv_lan 1') (4/2/2002) ver 1.0.1
*
******************************************************************/


#include <string>
#include <admin>
#include <adminlib>


/*****************************************************************
* Maximum number of Names/IDs/IPs in database. To increase this
* you'll need to recompile the plugin. It does use memory on the
* server, so you might want to experiment with it!
* I know how to do this dynamically in C, but not in the Small
* language. Any suggestons,
* email me please: jugger1@blueyonder.co.uk
*
* Also, this makes the files size for the plugin in large!
* MAX_ENTRIES 4000 makes it above 1 MB
*****************************************************************/


/* glodabl variable declarations */
#define MAX_ENTRIES	8000
#define MAX_NICKS	15	/* Maximum nicks to allow in database */

#define DELAY_TIME	5	/* delay for ID to be gained */
#define MAX_INFO_LENGTH	39	/* maximum length (in chars) of ID or IP */

#define MAX_ERRORS	5	/* max errors to be listed by the plugin */


new STRING_VERSION[] = "1.3";
new PLUGIN_NAME[] = "plugin_userinfo";

/* storage for player NAMEs and IDs/IPs in the database */
new DB_NAME[MAX_ENTRIES][MAX_NAME_LENGTH];
new DB_INFO[MAX_ENTRIES][MAX_INFO_LENGTH];
new DB_IPINFO[MAX_ENTRIES][MAX_INFO_LENGTH];

/* storage for player Names and IDs/IPs connected to the server */
new NAME[MAX_PLAYERS][MAX_NAME_LENGTH];
new INFO[MAX_PLAYERS][MAX_INFO_LENGTH];
new IPINFO[MAX_PLAYERS][MAX_INFO_LENGTH];

/* Greeting and timer flags */
new GREETED[MAX_PLAYERS];
new PL_TIMER[MAX_PLAYERS];

/* Database Files
1- LAN - IPs, ip_info.txt
2- Internet - SteamIDs and IPs, id_admin_info.txt 
3- Internet - SteamIDs, id_info.txt */
new FILE[3][] =	{"ip_info.txt", "id_info_admin.txt", "id_info.txt"};

/* You may change as the following
0 (Default)-- Only records usernames and SteamIDs.
1-- Records SteamIDs and IPs to id_info.txt.
2-- Records SteamIDs and IPs to id_info_admin.txt while also records just SteamIDs to id_info.txt.
*/
new RecordAdminInfo = 1;

/* others */
new Entries=0;

/* options */
new WelcomeMsg = 1;		/* 0 or 1 - greeting on or off */


/* Functions: Declarations */
forward DatabaseAdd(Index);
forward DatabaseCheck(UserIndex, iMatch);
forward DatabaseLoad();
forward RecordUser(Timer, WaitCount, RepeatCount, HLUser);
forward admin_showme(HLCommand, HLData, HLUserName, UserIndex);


/* Functions: Events */

/* called once on server start */
public plugin_init() {

	plugin_registerinfo("Jugger's ID/IP Collector (Fixed by Dark.MenacE)","Collects connecting player's name and ID (and/or IP)",STRING_VERSION);
	plugin_registercmd("admin_showme", "admin_showme", ACCESS_ALL, "admin_showme <target or ID or IP>: Shows all names on server for that ID or IP.");
		
	DatabaseLoad();	/* load database in to memory */

	return PLUGIN_CONTINUE;
}


/* called whenever a player changes name or connects */
public plugin_info(HLOldName, HLNewName, UserIndex) {
	new OldName[MAX_NAME_LENGTH];
	
	strinit(OldName);

	convert_string(HLNewName, NAME[UserIndex], MAX_NAME_LENGTH);
	convert_string(HLOldName, OldName, MAX_NAME_LENGTH);
		
	if (!strlen(OldName)) {
		strinit(INFO[UserIndex]);	/* initialise player data */
		strinit(IPINFO[UserIndex]);
		strinit(NAME[UserIndex]);
		GREETED[UserIndex] = 0;

		new Data[MAX_DATA_LENGTH];
		get_userIP(NAME[UserIndex], Data, MAX_DATA_LENGTH);
		strtok(Data, ":", IPINFO[UserIndex], MAX_DATA_LENGTH);	/* ..Store IP Without port */
	}

	PL_TIMER[UserIndex] = set_timer("RecordUser", 3, 1, NAME[UserIndex]);	/* Insert quickly on connect */
	return PLUGIN_CONTINUE;
}


/* called whenever a player disconnects -- Commented out because it was causing problems
Works fine without killing the timer, but left in, but commented out, JUST in case */
/* public plugin_disconnect(HLUserName, UserIndex) {	
	if (PL_TIMER[UserIndex]) {
		kill_timer(PL_TIMER[UserIndex]);
		PL_TIMER[UserIndex] = 0;
		return PLUGIN_CONTINUE;
	}
	return PLUGIN_CONTINUE;
} */



/* Add Name and Info to database for User
   Returns 1 on success, 0 otherwise */
DatabaseAdd(Index) {
	if ( strlen(INFO[Index])==0 || strlen(NAME[Index])==0 ) {	/* if no IP or ID, no addition to database made */
		return 0;
	}
	new Text[MAX_TEXT_LENGTH];
	
	if ( Entries<MAX_ENTRIES ) {	/* if Database is NOT full, add user to DB and save */
		strcpy(DB_NAME[Entries], NAME[Index], MAX_NAME_LENGTH);
		strcpy(DB_INFO[Entries], INFO[Index], MAX_INFO_LENGTH);
		get_userIP(NAME[Index], IPINFO[Index], MAX_INFO_LENGTH);
		
		if (RecordAdminInfo==1) {
			snprintf(Text, MAX_TEXT_LENGTH, "%s,%s,%s", INFO[Index], IPINFO[Index], NAME[Index]);
			writefile(FILE[RecordAdminInfo], Text);
		} else if (RecordAdminInfo==2) {
			snprintf(Text, MAX_TEXT_LENGTH, "%s,%s,%s", INFO[Index], IPINFO[Index], NAME[Index]);
			writefile(FILE[1], Text);
			snprintf(Text, MAX_TEXT_LENGTH, "%s,%s", INFO[Index], NAME[Index]);
			writefile(FILE[RecordAdminInfo], Text);
		} else {
			snprintf(Text, MAX_TEXT_LENGTH, "%s,%s", INFO[Index], NAME[Index]);
			writefile(FILE[RecordAdminInfo], Text);
		}
		Entries++;
	} else {
		snprintf(Text, MAX_TEXT_LENGTH, "%s: Database is full. No more names will be recorded.", PLUGIN_NAME);
		say(Text);
	}
	return 1;
}


/* Checks database to see if name/info has a matching entry
 * iMatch==0 - doesn't list matches of names with INFO
 * iMatch==1 - lists all names matched with INFO
 * Function returns 1 if details already exist in the database
 */
DatabaseCheck(UserIndex, iMatch) {
	new Count=0, Found=0;
	new Nick[MAX_NICKS][MAX_NAME_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	
	for (new i=0; i<Entries && Count<MAX_NICKS; i++) {
		/* Collect in 'Nick' array, all previous names used */
		if ( strcmp(DB_INFO[i], INFO[UserIndex])==0 ) {
			if ( strcasecmp(DB_NAME[i], NAME[UserIndex]) ) {
				strcpy(Nick[Count], DB_NAME[i], MAX_NAME_LENGTH);
				Count++;
			} else {
				Found = 1;
			}
		}
	}
	if ( iMatch==0 ) {
		if ( Count>=MAX_NICKS && Found==0 ) {	/* if maximum number of names reached, rename to first one used */
			snprintf(Text, MAX_TEXT_LENGTH, "%s: Maximum nicknames exceeded on: %s", PLUGIN_NAME, NAME[UserIndex]);
			say(Text);
			say("Renaming to first used name on the server.");
			snprintf(Text, MAX_TEXT_LENGTH, "name ^"%s^"", Nick[0]);
			execclient(NAME[UserIndex], Text);
			Found = 1;
		}
	} else {
		if ( Count > 0 ) {
			snprintf(Text, MAX_TEXT_LENGTH, "%s is also known as:", NAME[UserIndex]);
			selfmessage(Text);
			
			for (new i=0; i<Count; i++) {
				selfmessage(Nick[i]);
			}
		} else {
			snprintf(Text, MAX_TEXT_LENGTH, "%s has only this name on the database", NAME[UserIndex]);
			selfmessage(Text);
		}
	}
	return Found;
}


/* load database from server */
DatabaseLoad() {
	new i=0, iLine=0;
	new Text[MAX_TEXT_LENGTH];

	if (RecordAdminInfo == 0) {
		if ( fileexists(FILE[RecordAdminInfo])) {
			while ( readfile(FILE[RecordAdminInfo], Text, iLine++, MAX_TEXT_LENGTH) && i<MAX_ENTRIES ) {
				if ( strlen(Text)>0 ) {
					strsep(Text, ",", DB_INFO[i], MAX_INFO_LENGTH, DB_NAME[i], MAX_NAME_LENGTH);
					if (strlen(DB_NAME[i])) {
						i++;
					}
				}
			}
		} else {
			snprintf(Text, MAX_TEXT_LENGTH, "%s: file %s not found.", PLUGIN_NAME, FILE[RecordAdminInfo]);
			say(Text);
		}
	} else {
		if ( fileexists(FILE[1])) {
			while ( readfile(FILE[1], Text, iLine++, MAX_TEXT_LENGTH) && i<MAX_ENTRIES ) {
				if ( strlen(Text)>0 ) {
					strsep(Text, ",", DB_INFO[i], MAX_INFO_LENGTH, DB_IPINFO[i], MAX_TEXT_LENGTH, DB_NAME[i], MAX_NAME_LENGTH);
					if (strlen(DB_NAME[i])) {
						i++;
					}
				}
			}
		} else {
			snprintf(Text, MAX_TEXT_LENGTH, "%s: file %s not found.", PLUGIN_NAME, FILE[RecordAdminInfo]);
			say(Text);
		}
	}
	Entries = i;
}


/*
 * This function is delayed to allow for Authentication.
 * It is to greet the connected player on entry, then record
 * details.
 */
public RecordUser(Timer, WaitCount, RepeatCount, HLUser) {
	new UserIndex;
	new ID[MAX_INFO_LENGTH];
	new User[MAX_NAME_LENGTH];
	new Text[MAX_TEXT_LENGTH];

	convert_string(HLUser,User,MAX_NAME_LENGTH);

	if (!check_user(User)){ // invalid user
		kill_timer(Timer);
		PL_TIMER[UserIndex] = 0;
		return PLUGIN_HANDLED;
	}
	convert_string(HLUser, User, MAX_NAME_LENGTH);
	get_userindex(User, UserIndex);
	
	/* record SteamID */
	get_userAuthID(User, ID, MAX_INFO_LENGTH);
	
	if ( strncmp(ID, "STEAM_I", 7)==0 ) {	/* if ID not assigned, delay again */
		return PLUGIN_HANDLED;
	} else {
		strcpy(INFO[UserIndex], ID, MAX_INFO_LENGTH);
	}
	if ( WelcomeMsg==1 && GREETED[UserIndex]==0 ) {	/* display welcome message */
		snprintf(Text, MAX_TEXT_LENGTH, "Welcome, %s", User);
		messageex(User, Text, print_tty);
		GREETED[UserIndex] = 1;	/* greet only once per connect */
	}
	if ( DatabaseCheck(UserIndex, 0)==0 ) {	/* if details do not exist in database, add them */
		if ( DatabaseAdd(UserIndex)==0 ) {	/* if details fail to be recorded, repeat attempt */
			return PLUGIN_HANDLED;
		}
	}
	return PLUGIN_HANDLED;
}


/* Functions: User commands */

/* function for handling admin_showme command */
public admin_showme(HLCommand, HLData, HLUserName, UserIndex) {
	new Data[MAX_DATA_LENGTH];
	new TargetName[MAX_NAME_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	new User[MAX_NAME_LENGTH];
	new Index;

	convert_string(HLData, Data, MAX_DATA_LENGTH);
	convert_string(HLUserName, User, MAX_NAME_LENGTH);

	if ( check_user(Data)==1 ) {
		get_username(Data, TargetName, MAX_NAME_LENGTH);

		if ( check_immunity(TargetName)!=0 ) {
			snprintf(Text, MAX_TEXT_LENGTH, "Laf. %s is an admin. That's all private!", TargetName);
			selfmessage(Text);
		} else {
			get_userindex(TargetName, Index);
			strcpy(NAME[Index], TargetName, MAX_NAME_LENGTH);
			DatabaseCheck(Index, 1);
		}
	} else {
		selfmessage("Unrecognized player: ");
		selfmessage(Data);
	}
	return PLUGIN_HANDLED;
}
